#include <stdio.h>
#include "BitFieldTesters.h"

BitFieldTester :: BitFieldTester()
{
   // empty
}

BitFieldTester :: BitFieldTester(BMessage * from)
   : BArchivable(from)
{
   // empty
}

status_t BitFieldTester :: Archive(BMessage * into, bool deep) const
{
   status_t ret = BArchivable::Archive(into, deep);
   return ret;
} 

ConstantFieldTester :: ConstantFieldTester(bool result)
   : _result(result)
{
   // empty      
}
   
ConstantFieldTester :: ConstantFieldTester(BMessage * from)
   : BitFieldTester(from)
{
   if (from->FindBool("ctRes", &_result) != B_NO_ERROR) printf("ConstantFieldTester: Error, no ctRes!\n");
}
   
status_t ConstantFieldTester ::Archive(BMessage * into, bool deep) const
{
   status_t ret = BitFieldTester::Archive(into, deep);
      
   into->AddBool("ctRes", _result);
   return ret;
}
   
BArchivable * ConstantFieldTester ::Instantiate(BMessage * from)
{
   return (validate_instantiation(from, "ConstantFieldTester")) ? new ConstantFieldTester(from) : NULL;
}

bool ConstantFieldTester ::isMatching(uint32 field)
{
   return _result;
}
   
HasBitsFieldTester :: HasBitsFieldTester(uint32 requiredBits, uint32 forbiddenBits)
   : _requiredBits(requiredBits), _forbiddenBits(forbiddenBits)
{
   // empty      
}
   
HasBitsFieldTester :: HasBitsFieldTester(BMessage * from)
   : BitFieldTester(from)
{
   if (from->FindInt32("rqBits", (int32 *) &_requiredBits) != B_NO_ERROR) printf("HasBitsFieldTester: Error, no rqBits!\n");
   if (from->FindInt32("fbBits", (int32 *) &_forbiddenBits) != B_NO_ERROR) printf("HasBitsFieldTester: Error, no fbBits!\n");
}
   
status_t HasBitsFieldTester ::Archive(BMessage * into, bool deep) const
{
   status_t ret = BitFieldTester::Archive(into, deep);
      
   into->AddInt32("rqBits", _requiredBits);
   into->AddInt32("fbBits", _forbiddenBits);
   return ret;
}
   
BArchivable * HasBitsFieldTester :: Instantiate(BMessage * from)
{
   return (validate_instantiation(from, "HasBitsFieldTester")) ? new HasBitsFieldTester(from) : NULL;
}

bool HasBitsFieldTester :: isMatching(uint32 field)
{
   return ((((_requiredBits & (~field)) == 0)) && ((_forbiddenBits & (~field)) == _forbiddenBits));
}
   
NotFieldTester :: NotFieldTester(BitFieldTester * slave)
   : _slave(slave)
{
   // empty      
}
   
NotFieldTester :: ~NotFieldTester()
{
   delete _slave;
}
   
NotFieldTester :: NotFieldTester(BMessage * from)
   : BitFieldTester(from), _slave(NULL)
{
   BMessage slaveMsg;
   if (from->FindMessage("nSlave", &slaveMsg) == B_NO_ERROR)
   {
      BArchivable * slaveObj = instantiate_object(&slaveMsg);
      if (slaveObj)
      {
         _slave = dynamic_cast<BitFieldTester*>(slaveObj);
         if (_slave == NULL) 
         {
            printf("NotFieldTester: Error casting slaveObj to BitFieldTester!\n");
            delete slaveObj;
         }
      }
      else printf("NotFieldTester:  instantiate_object returned NULL!\n");
   }
   else printf("NotFieldTester: Couldn't unarchive NotFieldTester slave!\n");
}
   
status_t NotFieldTester :: Archive(BMessage * into, bool deep) const
{
   if (_slave == NULL) return B_ERROR;
      
   status_t ret = BitFieldTester::Archive(into, deep);

   if (ret == B_NO_ERROR)
   {
      BMessage msg;
      ret = _slave->Archive(&msg, deep);
      into->AddMessage("nSlave", &msg);
   }
      
   return ret;
}
   
BArchivable * NotFieldTester ::Instantiate(BMessage * from)
{
   return (validate_instantiation(from, "NotFieldTester")) ? new NotFieldTester(from) : NULL;
}

bool NotFieldTester ::isMatching(uint32 field)
{
   return _slave ? (!_slave->isMatching(field)) : false;
}
   
MinMatchFieldTester :: MinMatchFieldTester(int minNum, bool deleteSlaves)
   : _minNum(minNum), _deleteSlaves(deleteSlaves)  // _deleteSlaves state not archived!
{
   // empty      
}
   
MinMatchFieldTester :: ~MinMatchFieldTester()
{
   if (_deleteSlaves) for (int i=0; i<_slaves.CountItems(); i++) delete ((BitFieldTester *) _slaves.ItemAt(i));
}
   
MinMatchFieldTester :: MinMatchFieldTester(BMessage * from)
   : BitFieldTester(from), _deleteSlaves(true)
{
   int i=0;
   BMessage slaveMsg;
   while(from->FindMessage("mSlave", i++, &slaveMsg) == B_NO_ERROR)
   {
      BArchivable * slaveObj = instantiate_object(&slaveMsg);
      if (slaveObj)
      {
         BitFieldTester * nextSlave = dynamic_cast<BitFieldTester*>(slaveObj); 
         if (nextSlave) _slaves.AddItem(nextSlave);
         else
         {
            printf("MinMatchFieldTester: Error casting slaveObj to BitFieldTester!\n");
            delete slaveObj;
         }
      }
      else printf("MinMatchFieldTester:  instantiate_object returned NULL!\n");
   }
      
   if (from->FindInt32("mMin", (int32 *) &_minNum) != B_NO_ERROR) printf("MinMatchFieldTester: Error getting mMin!\n");
}
   
// (slave) should be allocated with new, becomes property of this object.
void MinMatchFieldTester :: AddSlave(const BitFieldTester * slave)
{
   _slaves.AddItem((void *) slave);
}
   
status_t MinMatchFieldTester :: Archive(BMessage * into, bool deep) const
{
   status_t ret = BitFieldTester::Archive(into, deep);
      
   if (ret == B_NO_ERROR)
   {
      for (int i=0; i<_slaves.CountItems(); i++)
      {
         BMessage msg;
         ret = ((BitFieldTester *)_slaves.ItemAt(i))->Archive(&msg, deep);
         if (ret != B_NO_ERROR) return ret;
         
         into->AddMessage("mSlave", &msg);
      }
   }
      
   into->AddInt32("mMin", _minNum);
   return ret;
}
   
BArchivable * MinMatchFieldTester :: Instantiate(BMessage * from)
{
   return (validate_instantiation(from, "MinMatchFieldTester")) ? new MinMatchFieldTester(from) : NULL;
}

// Returns true if at least (_minNum) slaves return true.
bool MinMatchFieldTester :: isMatching(uint32 field)
{
   if ((_minNum == 0)&&(_slaves.CountItems() == 0)) return true;  // 0 >= 0, so this should return true!
            
   int count = 0;
   for (int i=0; i<_slaves.CountItems(); i++)
      if ((((BitFieldTester *)_slaves.ItemAt(i))->isMatching(field)) && (++count >= _minNum)) return true;
   return false;
}
