#ifndef MetaKeyStateMap_h
#define MetaKeyStateMap_h

#include <support/SupportDefs.h>
#include <support/List.h>

class BitFieldTester;

/* This class defines a set of possible chording states (e.g.
 * "Left only", "Right only", "Both", "Either") for a meta-key (e.g. Shift),
 * and the description strings and qualifier bit-chords that go with them.
 */
class MetaKeyStateMap
{
public:
   MetaKeyStateMap();  
   // Note:  You MUST call SetInfo() directly after using this ctor!
   
   MetaKeyStateMap(const char * keyName);
   // Creates a MetaKeyStateMap with the give name (e.g. "Shift" or "Ctrl")
   
   ~MetaKeyStateMap();
   
   void SetInfo(const char * keyName);
   // For when you have to use the default ctor
   
   void AddState(const char * desc, const BitFieldTester * tester);
   // (tester) becomes property of this map!
   
   const char * GetName() const;
   // Returns the name of the meta-key (e.g. "Ctrl")
   
   int GetNumStates() const;
   // Returns the number of possible states contained in this MetaKeyStateMap.
   
   const BitFieldTester * GetNthStateTester(int stateNum) const;
   // Returns a BitFieldTester that tests for the nth state's presence.
   
   const char * GetNthStateDesc(int stateNum) const;   
   // Returns a textual description of the nth state (e.g. "Left")
   
private:  
   char * _keyName;    // e.g. "Alt" or "Ctrl"
   
   BList _stateDescs;   // list of strings e.g. "Left" or "Both"
   BList _stateTesters; // list of BitFieldTesters for testing bits of modifiers in state
};

#endif