#ifndef KeyCommandMap_h
#define KeyCommandMap_h

#include <support/List.h>
#include <support/Locker.h>

#include <app/Looper.h>
#include <app/Messenger.h>
#include <app/Message.h>
#include <app/MessageFilter.h>

// Maps BMessages to SpicyKeysSpecs!
// The thread here gets file update messages, and updates
// the _specs list to match them (asynchronously).
class KeyCommandMap : public BLooper
{
public:
   KeyCommandMap(const char * watchFile);
   ~KeyCommandMap();
   
   // Called when a key is pressed or depressed, in the input_server thread.
   // (keyMsg) is the KEY_DOWN message given by the Input Server.
   // (outlist) is a BList that additional BMessages can be added to (to insert events into the input stream)
   // (sendMessagesTo) is the address of a queue to send command messages to, for async effects.
   filter_result KeyEvent(const BMessage * keyMsg, BList * outlist, const BMessenger & sendMessagesTo);
 
   // Called whenever the a B_MOUSE_MOVED message is received.
   void MouseMoved(const BMessage * mouseMoveMsg);
   
   void MessageReceived(BMessage * msg);
   
   void DrainInjectedEvents(const BMessage * keyMsg, BList * outlist, const BMessenger & sendMessagesTo);

private:  
   void PutMessageToPort();
   void DeleteHKSList(BList * list);
   
   port_id _port;

   char * _fileName;
   
   BLocker _syncSpecs;  // locks the lists below   
   BList _injects;
   BList * _specs; 
   BMessage _lastMouseMessage;
};

#endif
