#include "SpicyKeysServerFilter.h"
#include "KeyCommandMap.h"
#include "CommandExecutor.h"

class BInputServerFilter;

// Called by input_server on startup.  Must be exported as a "C" function!
BInputServerFilter * instantiate_input_filter()
{
   return new SpicyKeysServerFilter;
}

SpicyKeysServerFilter :: SpicyKeysServerFilter()
  : _map(new KeyCommandMap("/boot/home/config/settings/spicykeys_settings")), _executor(new CommandExecutor)
{
   _messenger = BMessenger(_executor);
   _map->Run();
   _executor->Run();
}

SpicyKeysServerFilter :: ~SpicyKeysServerFilter()
{
   if (_map->Lock()) _map->Quit();
   if (_executor->Lock()) _executor->Quit();
}

status_t SpicyKeysServerFilter :: InitCheck()
{
   return B_NO_ERROR;
}

filter_result SpicyKeysServerFilter :: Filter(BMessage * msg, BList * outlist)
{
   filter_result ret = B_DISPATCH_MESSAGE;  // default
   
   switch(msg->what)
   {
      case B_KEY_DOWN: 
      case B_KEY_UP: 
      case B_UNMAPPED_KEY_DOWN:
      case B_UNMAPPED_KEY_UP:
         ret = _map->KeyEvent(msg, outlist, _messenger);
      break;
      
      case B_MOUSE_MOVED:  
      case B_MOUSE_UP:  
      case B_MOUSE_DOWN:  
         _map->MouseMoved(msg); 
      break;
   }
   _map->DrainInjectedEvents(msg, outlist, _messenger);
   return ret;
}
