#ifndef SpicyKeysSpec_h
#define SpicyKeysSpec_h

#include <app/Message.h>
#include <interface/Rect.h>
#include <interface/View.h>
#include <interface/Font.h>
#include <interface/Bitmap.h>

#include "CLVListItem.h"
#include "KeyInfos.h"

class CommandActuator;
class MetaKeyStateMap;

MetaKeyStateMap & GetNthKeyMap(int which);
void InitializeMetaMaps();

/* Objects of this class represent one hotkey "entry" in the preferences ListView.
 * Each SpicyKeysSpec contains the info necessary to generate both the proper GUI
 * display, and the proper BitFieldTester and CommandActuator object for the SpicyKeysCatcher
 * add-on to use.
 */
class SpicyKeysSpec : public CLVListItem
{
public:
   SpicyKeysSpec(const char * command);

   SpicyKeysSpec(const SpicyKeysSpec & copyMe);
   SpicyKeysSpec(BMessage *from);
   ~SpicyKeysSpec();

   virtual status_t Archive(BMessage *into, bool deep = true) const;
   static BArchivable *Instantiate(BMessage *from);
		
   void DrawItemColumn(BView* owner, BRect item_column_rect, int32 column_index, bool columnSelected, bool complete);
   void Update(BView *owner, const BFont *font);
   static int MyCompare(const CLVListItem* a_Item1, const CLVListItem* a_Item2, int32 KeyColumn);
   const char * GetCellText(int whichColumn) const;

   void SetCommand(const char * commandStr);
        
   // default layout of columns is set in here.
   enum {
      SHIFT_COLUMN_INDEX   = 0,
      CONTROL_COLUMN_INDEX = 1,
      COMMAND_COLUMN_INDEX = 2,
      OPTION_COLUMN_INDEX  = 3,   
      NUM_META_COLUMNS     = 4,  // shift, control, command, option, for now
      KEY_COLUMN_INDEX     = NUM_META_COLUMNS,
      STRING_COLUMN_INDEX  = 5
   };
        
   static const char * GetColumnName(int index);
   // Returns the name of the Nth Column.
        
   bool ProcessColumnKeyStroke(int whichColumn, const char * bytes, int32 key);
   // Update this spec's state in response to a keystroke to the given
   // column.  Returns true iff a change occurred.
        
   bool ProcessColumnMouseClick(int whichColumn);
   // Same as ProcessColumnKeyStroke, but for a mouse click instead.
       
   bool ProcessColumnTextString(int whichColumn, const char * string);
   // Same as ProcessColumnKeyStroke, but for a text string instead.
        
   virtual void Pulse(BView * owner);
        
private:
   void CacheViewFont(BView * owner);
   bool AttemptTabCompletion();
	    
   void UpdateIconBitmap();  // call this to ensure the icon is up-to-date
	    
   char * _command;        
   uint32 _commandLen;  // number of bytes in _command buffer
   uint32 _commandNul;  // index of the NUL byte in _commandBuffer        
   float _fTextOffset;
		
   BBitmap _bitmap;     // icon for associated program.  Invalid if none available.
   char * _lastBitmapName;
   bool _bitmapValid;
		
   uint32 _key;		
   int32 _metaCellStateIndex[NUM_META_COLUMNS];
		
   BPoint _cursorPt1;
   BPoint _cursorPt2;
   bool _cursorPtsValid;

   mutable char _scratch[50];
};

#endif

