#include <stdio.h>
#include <math.h>
#include <string.h>
#include <app/Application.h>

#include <interface/Alert.h>
#include <interface/Menu.h>
#include <interface/MenuItem.h>
#include <interface/MenuBar.h>
#include <interface/ScrollBar.h>
#include <interface/ScrollView.h>
#include <interface/Input.h>
#include <interface/PopUpMenu.h>

#include <storage/File.h>
#include <storage/Path.h>

#include "SpicyKeysWindow.h"
#include "SpicyKeysConstants.h"
#include "KeyInfos.h"
#include "SpicyKeysSpec.h"
#include "ParseCommandLine.h"
#include "MetaKeyStateMap.h"

// Window sizing constraints
#define MIN_WIDTH  600
#define MIN_HEIGHT 130
#define MAX_WIDTH  65535
#define MAX_HEIGHT 65535

// Default window position
#define WINDOW_START_X 30
#define WINDOW_START_Y 100

// Creates a pop-up-menu that reflects the possible states of the specified meta-key.
static BPopUpMenu * CreateMetaPopUp(int col);
static BPopUpMenu * CreateMetaPopUp(int col)
{
   MetaKeyStateMap & map = GetNthKeyMap(col);
   BPopUpMenu * popup = new BPopUpMenu(NULL, false);
   int numStates = map.GetNumStates();
   for (int i=0; i<numStates; i++) popup->AddItem(new BMenuItem(map.GetNthStateDesc(i), NULL));
   return popup;
}

// Creates a pop-up that allows the user to choose a key-cap visually
static BPopUpMenu * CreateKeysPopUp();
static BPopUpMenu * CreateKeysPopUp()
{
   BPopUpMenu * popup = new BPopUpMenu(NULL, false);
   int numKeys = GetNumKeyIndices();
   for (int i=0; i<numKeys; i++) 
   {
      const char * next = GetKeyName(i);
      if (next) popup->AddItem(new BMenuItem(next, NULL));
   }
   return popup;
}

SpicyKeysWindow::SpicyKeysWindow(BEntry * optSaveTo) : BWindow(BRect(WINDOW_START_X,WINDOW_START_Y,WINDOW_START_X+MIN_WIDTH,WINDOW_START_Y+MIN_HEIGHT*2),"SpicyKeys",B_DOCUMENT_WINDOW,0L), _keySetModified(false), _savePanel(NULL), _openPanel(NULL), _selectPanel(NULL), _lastOpenWasAppend(false)
{
   SetSizeLimits(MIN_WIDTH, MAX_WIDTH, MIN_HEIGHT, MAX_HEIGHT);
    
   BMenuBar * menuBar = new BMenuBar(BRect(0, 0, 0, 0), "Menu Bar");
    
   BMenu * fileMenu = new BMenu("File");
   fileMenu->AddItem(new BMenuItem("Open KeySet...",    new BMessage(J_OPEN_KEYSET), 'O'));
   fileMenu->AddItem(new BMenuItem("Append KeySet...",  new BMessage(J_APPEND_KEYSET), 'A'));
   fileMenu->AddItem(new BMenuItem("Revert to Saved", new BMessage(J_REVERT_KEYSET), 'A'));
   fileMenu->AddItem(new BSeparatorItem);
// fileMenu->AddItem(new BMenuItem("Save KeySet",       new BMessage(J_SAVE_KEYSET)));  // redundant w/button --jaf
   fileMenu->AddItem(new BMenuItem("Save Copy of KeySet As...", new BMessage(J_SAVE_KEYSET_AS), 'S'));
   fileMenu->AddItem(new BSeparatorItem);
   fileMenu->AddItem(new BMenuItem("About SpicyKeys",     new BMessage(J_ABOUT)));
   fileMenu->AddItem(new BSeparatorItem);
   fileMenu->AddItem(new BMenuItem("Quit",              new BMessage(B_QUIT_REQUESTED), 'Q'));
   menuBar->AddItem(fileMenu);
   
   AddChild(menuBar);
    
   font_height fh;
   be_plain_font->GetHeight(&fh);
   float vButtonHeight = ceil(fh.ascent)+ceil(fh.descent)+5.0f;
   
   BRect tableBounds = Bounds();
   tableBounds.top = menuBar->Bounds().bottom + 1;
   tableBounds.right -= B_V_SCROLL_BAR_WIDTH;
   tableBounds.bottom -= (B_H_SCROLL_BAR_HEIGHT + V_SPACING + vButtonHeight + V_SPACING*2);
   BScrollView* ContainerView;
   
   _columnListView = new ColumnListView(tableBounds,&ContainerView, NULL, B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS|B_NAVIGABLE,B_SINGLE_SELECTION_LIST,true,true,true,B_NO_BORDER);
   _columnListView->SetEditMessage(new BMessage(J_HOTKEY_ITEM_MODIFIED), BMessenger(this));
   const float metaWidth = 50.0f;
   for (int i=0; i < SpicyKeysSpec::NUM_META_COLUMNS; i++) _columnListView->AddColumn(new CLVColumn(SpicyKeysSpec::GetColumnName(i), CreateMetaPopUp(i), metaWidth, CLV_SORT_KEYABLE));
   _columnListView->AddColumn(new CLVColumn("Key", CreateKeysPopUp(), 60, CLV_SORT_KEYABLE));

   BPopUpMenu * popup = new BPopUpMenu(NULL, false);
   popup->AddItem(new BMenuItem("(Choose App with File Requester)", NULL));
   popup->AddItem(new BMenuItem("*InsertString \"Your Text Here\"", NULL));
   popup->AddItem(new BMenuItem("*MoveMouse +20 +0", NULL));
   popup->AddItem(new BMenuItem("*MoveMouseTo 50% 50%", NULL));
   popup->AddItem(new BMenuItem("*MouseButton 1", NULL));
   popup->AddItem(new BMenuItem("*LaunchHandler text/html", NULL));
   popup->AddItem(new BMenuItem("*Multi \"*MoveMouseTo 100% 0\" \"*MouseButton 1\"", NULL));
   popup->AddItem(new BMenuItem("*MouseDown", NULL));
   popup->AddItem(new BMenuItem("*MouseUp", NULL));   
   popup->AddItem(new BMenuItem("*SendMessage application/x-vnd.Be-TRAK 'Tfnd'", NULL));   
   popup->AddItem(new BMenuItem("*Beep", NULL));
   _columnListView->AddColumn(new CLVColumn("Application", popup, 323.0,CLV_SORT_KEYABLE));
   
   _columnListView->SetSortFunction(SpicyKeysSpec::MyCompare);
   AddChild(ContainerView);
   
   _columnListView->SetSelectionMessage(new BMessage(J_HOTKEY_ITEM_SELECTED));
   _columnListView->SetTarget(this);
   
   BRect buttonBounds = Bounds();
   buttonBounds.left += V_SPACING;
   buttonBounds.right = ((buttonBounds.right - buttonBounds.left) / 2.0f) + buttonBounds.left;   
   buttonBounds.bottom -= V_SPACING * 2;
   buttonBounds.top = buttonBounds.bottom - vButtonHeight;   
   buttonBounds.right -= B_V_SCROLL_BAR_WIDTH;
   float origRight = buttonBounds.right;
   buttonBounds.right = (buttonBounds.left + origRight)*0.40f - (V_SPACING / 2);      
   AddChild(_addButton = new JResizableButton(Bounds(), buttonBounds, "add", "Add New SpicyKey", new BMessage(J_ADD_HOTKEY_ITEM)));
   buttonBounds.left = buttonBounds.right + V_SPACING;
   buttonBounds.right = origRight;
   AddChild(_removeButton = new JResizableButton(Bounds(), buttonBounds, "remove", "Remove Selected SpicyKey", new BMessage(J_REMOVE_HOTKEY_ITEM)));
   _removeButton->SetEnabled(false);
   
   float offset = (buttonBounds.right - buttonBounds.left) / 2.0f;
   BRect saveButtonBounds = buttonBounds;
   saveButtonBounds.right = Bounds().right - B_V_SCROLL_BAR_WIDTH - offset;
   saveButtonBounds.left  = buttonBounds.right + V_SPACING + offset;
   AddChild(_saveButton = new JResizableButton(Bounds(), saveButtonBounds, "save", "Save & Apply", new BMessage(J_SAVE_KEYSET)));
   _saveButton->SetEnabled(false);
   
   if (optSaveTo != NULL)
   {
         BMessage msg(B_REFS_RECEIVED);
         entry_ref ref;
         if (optSaveTo->GetRef(&ref) == B_NO_ERROR)
         {
            msg.AddRef("refs", &ref);
            msg.AddString("startupRef", "please");
            PostMessage(&msg);   // Tell ourself to load this file if it exists.            
         }
   }
   
   Show();
}


SpicyKeysWindow::~SpicyKeysWindow()
{
    delete _savePanel;
    delete _openPanel;
    delete _selectPanel;
    be_app->PostMessage(B_QUIT_REQUESTED);
}

bool SpicyKeysWindow :: QuitRequested()
{
    bool ret = true;
     
    if (_keySetModified)
    {
       BAlert * alert = new BAlert("SpicyKeys warning", "Really quit without saving your changes?", "Don't Save", "Cancel", "Save");
       switch(alert->Go())
       {
          case 0:
             // Okay to quit
             break;
          
          case 1:
             ret = false;
             break;
           
          case 2:
             // Save:  automatically if possible, otherwise go back and open up the file requester
             if (_lastSaved.InitCheck() == B_NO_ERROR)
             {
                if (SaveKeySet(_lastSaved) == false) (new BAlert("SpicyKeys Error", "SpicyKeys was unable to save your KeySet file!", "Oh no"))->Go();
             }
             else
             {
                PostMessage(J_SAVE_KEYSET);  
                ret = false;
             }             
             break;          
       }
    }    
    if (ret) _columnListView->DeselectAll();  // avoid mysterious crash on PPC!?
    return ret;
}

// Saves a settings file to (saveEntry).  Returns true iff successful.
bool SpicyKeysWindow :: SaveKeySet(BEntry & saveEntry)
{
   BFile saveTo(&saveEntry, B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
   if (saveTo.InitCheck() != B_NO_ERROR) return false;
   
   BMessage saveMsg;
   for (int i=0; i<_columnListView->CountItems(); i++) 
   {
      BMessage next;
      if (((SpicyKeysSpec *)_columnListView->ItemAt(i))->Archive(&next) == B_NO_ERROR) saveMsg.AddMessage("spec", &next);
                                                                                  else printf("Error archiving SpicyKeysSpec #%i!\n",i);                                                                               
   }
   bool ret = (saveMsg.Flatten(&saveTo) == B_NO_ERROR);
   if (ret) 
   {
//    _lastSaved = saveEntry;  // don't execute this line--I want the system file to remain the default -- jaf      
      _keySetModified = false;
      _saveButton->SetEnabled(false);
   }

   return ret;
}

// Appends new entries from the file specified in the "spec" entry of (loadMsg).  Returns true iff successful.
bool SpicyKeysWindow :: LoadKeySet(const BMessage & loadMsg)
{
    int i=0;
    BMessage next;
    while(loadMsg.FindMessage("spec", i++, &next) == B_NO_ERROR)
    {
        SpicyKeysSpec * spec = (SpicyKeysSpec *) SpicyKeysSpec :: Instantiate(&next);
        if (spec != NULL) _columnListView->AddItem(spec);
                     else printf("LoadKeySet:  Error parsing spec!\n");
    }
    return true;
}

// Creates a new entry and adds it to the GUI.  (defaultCommand) will be the text in the entry, or NULL if no text is desired.
void SpicyKeysWindow :: AddNewSpec(const char * defaultCommand)
{
   MarkKeySetModified();

   SpicyKeysSpec * spec;      
   int curSel = _columnListView->CurrentSelection();
   if (curSel >= 0)
   {   
      spec = new SpicyKeysSpec(*((SpicyKeysSpec *) _columnListView->ItemAt(curSel)));  // copy this...
      if (defaultCommand) spec->SetCommand(defaultCommand);
   }
   else spec = new SpicyKeysSpec(defaultCommand ? defaultCommand : "");
   
   _columnListView->AddItem(spec);
   _columnListView->Select(_columnListView->CountItems()-1);
   _columnListView->ScrollToSelection();
}

void SpicyKeysWindow :: MessageReceived(BMessage * msg)
{
    switch(msg->what) 
    {
       case J_OPEN_KEYSET: 
       case J_APPEND_KEYSET: 
         _lastOpenWasAppend = (msg->what == J_APPEND_KEYSET);
         if (_openPanel) _openPanel->Show();
         else 
         {
            BMessenger m(this);
           _openPanel = new BFilePanel(B_OPEN_PANEL, &m, NULL, 0, false);
           _openPanel->Show();
         }
         _openPanel->SetButtonLabel(B_DEFAULT_BUTTON, _lastOpenWasAppend ? "Append" : "Open");
       break;
       
       case J_REVERT_KEYSET:
       {
          // Send a message to myself, to get me to reload the settings file
          _lastOpenWasAppend = false;
          BMessage reload(B_REFS_RECEIVED);
          entry_ref eref;
          BEntry(SPICYKEYS_SETTINGS_FILE, true).GetRef(&eref);
          reload.AddRef("refs", &eref);
          reload.AddString("startupRef", "yeah");
          PostMessage(&reload);
       }
       break;

       // Respond to drag-and-drop messages here       
       case B_SIMPLE_DATA:
       {       
          int i = 0;
          
          entry_ref ref;
          while(msg->FindRef("refs", i++, &ref) == B_NO_ERROR)
          {
              BEntry entry(&ref);
              if (entry.InitCheck() == B_NO_ERROR)
              {
                 BPath path(&entry);
                 if (path.InitCheck() == B_NO_ERROR)
                 {                    
                    // Add a new item with the given path.
                    BString str(path.Path());
                    DoStandardEscapes(str);
                    AddNewSpec(str.String());                    
                 }
              }
          }
       } 
       break;
       
       // Respond to FileRequester's messages here
       case B_REFS_RECEIVED:
       {
         // Find file ref                  
         entry_ref ref;
         bool isStartMsg = msg->HasString("startupRef");
         if (msg->FindRef("refs", &ref) == B_NO_ERROR)
         {
            // load the file into (fileMsg)
            BMessage fileMsg;
          {
             BFile file(&ref, B_READ_ONLY);
             if ((file.InitCheck() != B_NO_ERROR)||(fileMsg.Unflatten(&file) != B_NO_ERROR))
              {
                 if (isStartMsg)
                 {
                    // use this to save to anyway                    
                    _lastSaved = BEntry(&ref);
                    break;
                 }
                 else
                 {
                     BAlert *error = new BAlert("SpicyKeys Error", "SpicyKeys was unable to open your KeySet file!", "Okay");
                     error->Go(NULL);  // go() will delete (error) when it returns
                     break;
                  }
               }
          }
 
            if (_lastOpenWasAppend == false)
            {
               // Clear the menu...
               SpicyKeysSpec * item;
               do {delete (item = ((SpicyKeysSpec*)_columnListView->RemoveItem(int32(0))));} while(item);
            }
         
          if (LoadKeySet(fileMsg))
          {
                if (isStartMsg) _lastSaved = BEntry(&ref);
                _saveButton->SetEnabled(isStartMsg == false);
                
                // If we just loaded in the spicykeys settings file, then no need to tell the user to save on exit.
                entry_ref eref;
                BEntry(SPICYKEYS_SETTINGS_FILE, true).GetRef(&eref);
                if (ref == eref) _keySetModified = false;
          }
          else
          {
                BAlert *error = new BAlert("SpicyKeys Error", "SpicyKeys was unable to parse your KeySet file!", "Okay");
                error->Go(NULL);  // go() will delete (error) when it returns
                break;          
          }
         }
       }
       break;
       
       // These messages come from the pop-up menu of the Applications column
       case J_SELECT_APPLICATION:
       {
          int csel = _columnListView->CurrentSelection();
          if (csel >= 0)
          {          
             entry_ref aref;
             if (msg->FindRef("refs", &aref) == B_NO_ERROR)
             {
                BEntry ent(&aref);
                if (ent.InitCheck() == B_NO_ERROR)
                {
                   BPath path;
                   if ((ent.GetPath(&path) == B_NO_ERROR)&&
                       (((SpicyKeysSpec *)_columnListView->ItemAt(csel))->ProcessColumnTextString(SpicyKeysSpec::STRING_COLUMN_INDEX, path.Path())))
                   {
                      _columnListView->InvalidateItem(csel);  // cause repaint of row
                      MarkKeySetModified();
                   }
                }
             }
          }
       }
       break;
       
       case J_SAVE_KEYSET:
       {
         bool showSaveError = false;
         
         const char * name;
         entry_ref entry;
         if ((msg->FindString("name", &name) == B_NO_ERROR)&&(msg->FindRef("directory", &entry) == B_NO_ERROR))
         {
            BDirectory dir(&entry);
            BEntry saveTo(&dir, name, true);
            showSaveError = ((saveTo.InitCheck() != B_NO_ERROR)||(SaveKeySet(saveTo) == false));
         }
         else if (_lastSaved.InitCheck() == B_NO_ERROR)
         {
            // We've saved this before, save over previous file.
            showSaveError = (SaveKeySet(_lastSaved) == false);
         }
         else PostMessage(J_SAVE_KEYSET_AS);  // open the save requester...
         
         if (showSaveError) 
         {
            BAlert *error = new BAlert("SpicyKeys Error", "SpicyKeys wasn't able to save your keyset.", "Okay");
            error->Go(NULL);
         }
       }
       break;

       case J_SAVE_KEYSET_AS:
       {
         if (_savePanel) _savePanel->Show();
          else
          {
             BMessage msg(J_SAVE_KEYSET);
             BMessenger messenger(this);
            _savePanel = new BFilePanel(B_SAVE_PANEL, &messenger, NULL, 0, false, &msg);
            _savePanel->Show();
          }
      }
       break;
       
       case J_ABOUT: 
          (new BAlert("About SpicyKeys", SPICYKEYS_ABOUT_STRING, "Hmmm"))->Go(NULL);
       break;
         
       case J_ADD_HOTKEY_ITEM:      
          AddNewSpec(NULL);   
       break;
       
       case J_REMOVE_HOTKEY_ITEM:
       {
          int index = _columnListView->CurrentSelection();
          if (index >= 0)
          {
             CLVListItem * item = (CLVListItem *) _columnListView->ItemAt(index);
             _columnListView->RemoveItem(index);          
             delete item;
             MarkKeySetModified();

             // Rules for new selection:  If there is an item at (index), select it.
             // Otherwise, if there is an item at (index-1), select it.
             // Otherwise, select nothing.
             int num = _columnListView->CountItems();
             if (num > 0)
             {
                if (index < num) _columnListView->Select(index);
                else 
                {
                   if (index > 0) index--;
                   if (index < num) _columnListView->Select(index);
                }
             }
          }
       }
       break;
       
       // Received when the user clicks on the ColumnListView
       case J_HOTKEY_ITEM_SELECTED:
       {
          int32 index = -1;
          msg->FindInt32("index", &index);
          bool validItem = (index >= 0);
          _removeButton->SetEnabled(validItem);
       }
       break;
       
       // Received when an entry is to be modified in response to GUI activity
       case J_HOTKEY_ITEM_MODIFIED:       
       {
          int32 row, column;
          
          if ((msg->FindInt32("row", &row) == B_NO_ERROR)&&(msg->FindInt32("column", &column) == B_NO_ERROR))
          {
             int32 key;
             const char * bytes;
             
             if (row >= 0)
             {
                SpicyKeysSpec * item = (SpicyKeysSpec *) _columnListView->ItemAt(row);
                bool repaintNeeded = false;  // default
                
                if (msg->HasInt32("mouseClick"))
                {
                   repaintNeeded = item->ProcessColumnMouseClick(column);
                }                
                else if ((msg->FindString("bytes", &bytes) == B_NO_ERROR)&&(msg->FindInt32("key", &key) == B_NO_ERROR))
                {             
                   repaintNeeded = item->ProcessColumnKeyStroke(column, bytes, key);
                }
                else if (msg->FindInt32("unmappedkey", &key) == B_NO_ERROR)
                {
                   repaintNeeded = ((column == item->KEY_COLUMN_INDEX)&&((key > 0xFF)||(GetKeyName(key) != NULL))&&(item->ProcessColumnKeyStroke(column, NULL, key)));
                }  
                else if (msg->FindString("text", &bytes) == B_NO_ERROR)
                {
                   if ((bytes[0] == '(')&&(bytes[1] == 'C'))
                   {
                      if (_selectPanel) _selectPanel->Show();
                      else 
                      {
                         BMessage msg(J_SELECT_APPLICATION);
                         BMessenger m(this);
                        _selectPanel = new BFilePanel(B_OPEN_PANEL, &m, NULL, 0, false, &msg);
                        _selectPanel->Show();
                      }
                      _selectPanel->SetButtonLabel(B_DEFAULT_BUTTON, "Select");
                   }
                   else repaintNeeded = item->ProcessColumnTextString(column, bytes);
                }
               
                if (repaintNeeded)
                {
                   _columnListView->InvalidateItem(row);  // cause repaint of row
                   MarkKeySetModified();
                }
             }
          }                 
       }
       break;
       
       default:
          BWindow::MessageReceived(msg);
       break;
    }    
}


void SpicyKeysWindow :: MarkKeySetModified()
{
    if (_keySetModified == false) 
    {
       _keySetModified = true;
       _saveButton->SetEnabled(true);
    }             
}

void SpicyKeysWindow::Quit()
{
    for (int i=_columnListView->CountItems()-1; i>=0; i--) delete (SpicyKeysSpec *)_columnListView->ItemAt(i);
    _columnListView->MakeEmpty();   
    BWindow::Quit();
}


void SpicyKeysWindow :: FrameResized(float w, float h)
{
    _addButton->ChangeToNewSize(w,h);
    _removeButton->ChangeToNewSize(w,h);
    _saveButton->ChangeToNewSize(w,h);
}
