#ifndef SpicyKeysWindow_h
#define SpicyKeysWindow_h

#include <app/Message.h>
#include <interface/Window.h>
#include <interface/Point.h>

#include <storage/Entry.h>
#include <storage/FilePanel.h>

#include "ColumnListView.h"
#include "JResizableButton.h"

/** This class defines our preferences/configuration window. */
class SpicyKeysWindow : public BWindow
{
public:
   SpicyKeysWindow(BEntry * optSaveTo = NULL);
   // If (optSaveTo) is non-NULL, settings will be loaded from the file it represents.
   
   ~SpicyKeysWindow();

   virtual void Quit();
   // Overridden to do some cleanup
   
   virtual void FrameResized(float w, float h);
   // Overridden to resize the add/remove buttons to reflect the new window size.
        
   virtual void MessageReceived(BMessage * msg);
       
   // BMessage 'what' codes, representing commands understood by this Window.
   enum {
      J_ADD_HOTKEY_ITEM = 'SpKy',   // Add a new hotkey entry to the GUI list
      J_REMOVE_HOTKEY_ITEM,         // Remove a hotkey entry from the GUI list
      J_HOTKEY_ITEM_SELECTED,       // Give the "focus bar" to the specified entry
      J_HOTKEY_ITEM_MODIFIED,       // Update the state of an entry to reflect user's changes
      J_OPEN_KEYSET,                // Bring up a File requester to load new settings
      J_APPEND_KEYSET,              // Bring up a File requester to append settings
      J_REVERT_KEYSET,              // Dump the current state and re-read settings from disk
      J_SAVE_KEYSET,                // Save the current settings to disk
      J_SAVE_KEYSET_AS,             // Bring up a File requester to save current settings
      J_SELECT_APPLICATION,         // Set the current entry to point to the given file
      J_ABOUT                       // Show the About Box
   };
        
   virtual bool QuitRequested();
   // Overridden to bring up the "Really Quit without saving changes?" requester
               
   private:
     BMenuItem * CreateActuatorPresetMenuItem(const char * label) const;
     void AddNewSpec(const char * defaultCommand);
     void MarkKeySetModified();
     bool LoadKeySet(const BMessage & loadMsg);
     bool SaveKeySet(BEntry & saveEntry);

     JResizableButton * _addButton;
     JResizableButton * _removeButton;
     JResizableButton * _saveButton;
       
     ColumnListView* _columnListView;

     BFilePanel * _savePanel;   // for saving settings
     BFilePanel * _openPanel;   // for loading settings
     BFilePanel * _selectPanel; // for selecting apps to launch
        
     BEntry _lastSaved;        // Points to the settings file to save to
     bool _keySetModified;     // true iff changes were made since last load or save
     bool _lastOpenWasAppend;  // true iff the file-requester's ref should be appended to current
};

#endif

