#include <stdio.h>
#include <kernel/OS.h>
#include <app/Messenger.h>
#include "CommandActuators.h"
#include "SpicyKeysConstants.h"
#include "SpicyKeyCatcher/SpicyKeysFilterConstants.h"

int main(int argc, char ** argv)
{
   if (argc < 2)
   {
      printf("usage:  dospicy '*InsertString hi there!'\n");
      return 5;
   }
  
   // Assemble the command string
   BString s;
   for (int i=1; i<argc; i++) 
   {
      if (strlen(argv[i]) > 0)
      {
         if (s.Length() > 0) s += ' ';
         s += argv[i];
      }
   }
    
   CommandActuator * ca = CreateCommandActuator(s.String());
   if (ca)
   {
      BMessage arc;
      if (ca->Archive(&arc) == B_NO_ERROR)
      {
         port_id port = find_port(SPICYKEYS_CATCHER_PORT_NAME);
         if (port >= 0)
         { 
            // Read a BMessage from the port to get our BMessenger
            char buf[2048];
            int32 code;
            ssize_t numBytes = read_port_etc(port, &code, buf, sizeof(buf), B_TIMEOUT, 250000);
            if (numBytes > 0) 
            {
               BMessage msg;
               if (msg.Unflatten(buf) == B_NO_ERROR)
               {
                  BMessenger target;
                  if (msg.FindMessenger("target", &target) == B_NO_ERROR)
                  {
                     BMessage replenish(J_REPLENISH_MESSENGER);
                     if (target.SendMessage(&replenish) == B_NO_ERROR)
                     {
                        BMessage doIt(J_EXECUTE_COMMAND);
                        doIt.AddMessage("act", &arc);
                        if (target.SendMessage(&doIt) == B_NO_ERROR)
                        {
                           // success!!
                           delete ca;
                           return 0;
                        }
                        else printf("Couldn't send doIt BMessage!\n");
                     }
                     else printf("Couldn't send replenish BMessage!\n");
                  }
                  else printf("Couldn't find BMessenger in message!\n");
               }
               else printf("Error parsing BMessage from port!\n");
msg.PrintToStream();
            }
            else printf("Error reading data from port!\n"); 
         }
         else printf("Error, couldn't find SpicyKeysCatcher port.  Is SpicyKeys 1.26+ installed and running?\n");
      }
      else printf("Error, couldn't archive CommandActuator!\n");

      delete ca;
   }
   else printf("Error, couldn't create Command Actuator for [%s]\n", s.String());

   return 10;
}
